@@ -223,9 +223,11 @@ bool endsWithSeperatedKeyword(const QString& str, const QString& shouldEndWith)
 
 QString ExpressionParser::popExpression(ExpressionParser::Status* status)
 {
-    QString operatingOn = getRemainingCode().trimmed().replace('\t', ' ');
+    const auto remaining = getRemainingCode();
+    auto trimmed = remaining.trimmed();
+    auto operatingOn = trimmed.replace('\t', ' ');
     bool lineIsEmpty = false;
-    for ( QString::const_iterator it = getRemainingCode().constEnd()-1; it != getRemainingCode().constEnd(); it-- ) {
+    for ( auto it = remaining.constEnd()-1; it != remaining.constEnd(); it-- ) {
         if ( ! it->isSpace() ) {
             break;
         }
@@ -284,7 +286,6 @@ QString ExpressionParser::popExpression(ExpressionParser::Status* status)
     KTextEditor::Cursor startCursor;
     QString expr = CodeHelpers::expressionUnderCursor(f, KTextEditor::Cursor(lastLine, f.fetchLine(lastLine).length() - 1),
                                                       startCursor, true);
-    Q_UNUSED(startCursor);
     if ( expr.isEmpty() ) {
         *status = NothingFound;
     }
@@ -318,10 +319,6 @@ void createArgumentList(Declaration* dec_, QString& ret, QList< QVariant >* high
         if (DUChainUtils::getArgumentContext(dec))
             parameters = DUChainUtils::getArgumentContext(dec)->localDeclarations();
 
-        uint defaultParamNum = 0;
-
-        int firstDefaultParam = parameters.count() - decl->defaultParametersSize();
-
         ret = '(';
         bool first = true;
         int num = 0;
@@ -331,7 +328,10 @@ void createArgumentList(Declaration* dec_, QString& ret, QList< QVariant >* high
             // the function is a class method, and its first argument is "self". Don't display that.
             skipFirst = true;
         }
-        
+
+        uint defaultParamNum = 0;
+        int firstDefaultParam = parameters.count() - decl->defaultParametersSize() - skipFirst;
+
         // disable highlighting when in default arguments, it doesn't make much sense then
         bool disableHighlighting = false;
         
