@@ -31,12 +31,22 @@ public:
     PythonDeclarationCompletionItem(KDevelop::DeclarationPointer decl = KDevelop::DeclarationPointer(), 
                                     QExplicitlySharedDataPointer<KDevelop::CodeCompletionContext> context = QExplicitlySharedDataPointer<KDevelop::CodeCompletionContext>(), 
                                     int inheritanceDepth = 0);
-    virtual QVariant data(const QModelIndex& index, int role, const KDevelop::CodeCompletionModel* model) const override;
+    QVariant data(const QModelIndex& index, int role, const KDevelop::CodeCompletionModel* model) const override;
     QString shortenedTypeString(KDevelop::DeclarationPointer decl, int desiredTypeLength) const override;
 
     void setTypeHint(PythonCodeCompletionContext::ItemTypeHint type);
     void addMatchQuality(int add);
 
+    bool createsExpandingWidget() const override
+    {
+        return true;
+    }
+
+    QWidget* createExpandingWidget(const CodeCompletionModel* /*model*/) const override
+    {
+        return new Python::NavigationWidget(m_declaration, {}, {}, {}, KDevelop::AbstractNavigationWidget::EmbeddableWidget);
+    }
+
 protected:
     PythonCodeCompletionContext::ItemTypeHint m_typeHint;
     int m_addMatchQuality;
