@@ -20,12 +20,11 @@
  */
 
 #include "declarationbuilder.h"
-#include "duchain/declarations/decorator.h"
 #include "duchain/declarations/functiondeclaration.h"
-#include "duchain/declarations/classdeclaration.h"
 
 #include "types/hintedtype.h"
 #include "types/unsuretype.h"
+#include "types/nonetype.h"
 #include "types/indexedcontainer.h"
 #include "contextbuilder.h"
 #include "expressionvisitor.h"
@@ -34,6 +33,7 @@
 #include "assistants/missingincludeassistant.h"
 #include "correctionhelper.h"
 
+#include <language/duchain/classdeclaration.h>
 #include <language/duchain/functiondeclaration.h>
 #include <language/duchain/declaration.h>
 #include <language/duchain/duchain.h>
@@ -66,7 +66,6 @@ DeclarationBuilder::DeclarationBuilder(Python::PythonEditorIntegrator* editor, i
     , m_ownPriority(ownPriority)
 {
     setEditor(editor);
-    qCDebug(KDEV_PYTHON_DUCHAIN) << "Building Declarations";
 }
 
 DeclarationBuilder:: ~DeclarationBuilder()
@@ -92,17 +91,16 @@ ReferencedTopDUContext DeclarationBuilder::build(const IndexedString& url, Ast*
     // The declaration builder needs to run twice, so it can resolve uses of e.g. functions
     // which are called before they are defined (which is easily possible, due to python's dynamic nature).
     if ( ! m_prebuilding ) {
-        qCDebug(KDEV_PYTHON_DUCHAIN) << "building, but running pre-builder first";
         DeclarationBuilder* prebuilder = new DeclarationBuilder(editor(), m_ownPriority);
         prebuilder->m_currentlyParsedDocument = currentlyParsedDocument();
         prebuilder->setPrebuilding(true);
         prebuilder->m_futureModificationRevision = m_futureModificationRevision;
         updateContext = prebuilder->build(url, node, updateContext);
-        qCDebug(KDEV_PYTHON_DUCHAIN) << "pre-builder finished";
         delete prebuilder;
+        qCDebug(KDEV_PYTHON_DUCHAIN) << "Second declarationbuilder pass";
     }
     else {
-        qCDebug(KDEV_PYTHON_DUCHAIN) << "prebuilding";
+        qCDebug(KDEV_PYTHON_DUCHAIN) << "Prebuilding declarations";
     }
     return DeclarationBuilderBase::build(url, node, updateContext);
 }
@@ -149,9 +147,7 @@ template<typename T> T* DeclarationBuilder::visitVariableDeclaration(Ast* node,
         // this prevents "bar" from being declared in something like "foo = bar"
         // This is just a sanity check, the code should never request creation of a variable
         // in such cases.
-        QList<ExpressionAst::Context> declaringContexts;
-        declaringContexts << ExpressionAst::Store << ExpressionAst::Parameter << ExpressionAst::AugStore;
-        if ( ! declaringContexts.contains(currentVariableDefinition->context) ) {
+        if ( currentVariableDefinition->context != ExpressionAst::Context::Store ) {
             return 0;
         }
         Identifier* id = currentVariableDefinition->identifier;
@@ -168,18 +164,10 @@ template<typename T> T* DeclarationBuilder::visitVariableDeclaration(Ast* node,
 
 QList< Declaration* > DeclarationBuilder::existingDeclarationsForNode(Identifier* node)
 {
-    QList<Declaration*> existingDeclarations = currentContext()->findDeclarations(
+    return currentContext()->findDeclarations(
         identifierForNode(node).last(), CursorInRevision::invalid(), 0,
         (DUContext::SearchFlag) (DUContext::DontSearchInParent | DUContext::DontResolveAliases)
     );
-    // append arguments context
-    if ( m_mostRecentArgumentsContext ) {
-        QList<Declaration*> args = m_mostRecentArgumentsContext->findDeclarations(
-            identifierForNode(node).last(), CursorInRevision::invalid(), 0, DUContext::DontSearchInParent
-        );
-        existingDeclarations.append(args);
-    }
-    return existingDeclarations;
 }
 
 DeclarationBuilder::FitDeclarationType DeclarationBuilder::kindForType(AbstractType::Ptr type, bool isAlias)
@@ -316,7 +304,6 @@ template<typename T> T* DeclarationBuilder::visitVariableDeclaration(Identifier*
         if ( currentContext()->type() == DUContext::Function ) {
             // check for argument type hints (those are created when calling functions)
             AbstractType::Ptr hints = Helper::extractTypeHints(dec->abstractType());
-            qCDebug(KDEV_PYTHON_DUCHAIN) << hints->toString();
             if ( hints.cast<IndexedContainer>() || hints.cast<ListType>() ) {
                 // This only happens when the type hint is a tuple, which means the vararg/kwarg of a function is being processed.
                 newType = hints;
@@ -834,6 +821,7 @@ void DeclarationBuilder::visitYield(YieldAst* node)
     if ( auto previous = t->returnType().cast<ListType>() ) {
         // If the return type of the function already is set to a list, *add* the encountered type
         // to its possible content types.
+        DUChainWriteLocker lock;
         previous->addContentType<Python::UnsureType>(encountered);
         t->setReturnType(previous.cast<AbstractType>());
     }
@@ -887,7 +875,7 @@ void DeclarationBuilder::applyDocstringHints(CallAst* node, FunctionDeclaration:
     QStringList args;
     QHash< QString, std::function<void()> > items;
     items["addsTypeOfArg"] = [&]() {
-        const int offset = ! args.isEmpty() ? args.at(0).toInt() : 0;
+        const int offset = ! args.isEmpty() ? (int) args.at(0).toUInt() : 0;
         if ( node->arguments.length() <= offset ) {
             return;
         }
@@ -904,7 +892,7 @@ void DeclarationBuilder::applyDocstringHints(CallAst* node, FunctionDeclaration:
         v.lastDeclaration()->setType(container);
     };
     items["addsTypeOfArgContent"] = [&]() {
-        const int offset = ! args.isEmpty() ? args.at(0).toInt() : 0;
+        const int offset = ! args.isEmpty() ? (int) args.at(0).toUInt() : 0;
         if ( node->arguments.length() <= offset ) {
             return;
         }
@@ -1005,8 +993,6 @@ void DeclarationBuilder::addArgumentTypeHints(CallAst* node, DeclarationPointer
     {
         atVararg = atVararg || currentParamIndex == function->vararg(); // Not >=, nonexistent vararg is -1.
 
-        qCDebug(KDEV_PYTHON_DUCHAIN) << currentParamIndex << currentArgumentIndex << atVararg << function->vararg();
-
         ExpressionAst* arg = node->arguments.at(currentArgumentIndex);
 
         ExpressionVisitor argumentVisitor(currentContext());
@@ -1026,7 +1012,6 @@ void DeclarationBuilder::addArgumentTypeHints(CallAst* node, DeclarationPointer
             indexInVararg++;
             Declaration* parameter = parameters.at(function->vararg());
             IndexedContainer::Ptr varargContainer = parameter->type<IndexedContainer>();
-            qCDebug(KDEV_PYTHON_DUCHAIN) << "adding" << addType->toString() << "at position" << indexInVararg;
             if ( ! varargContainer ) continue;
             if ( varargContainer->typesCount() > indexInVararg ) {
                 AbstractType::Ptr oldType = varargContainer->typeAt(indexInVararg).abstractType();
@@ -1095,6 +1080,7 @@ void DeclarationBuilder::addArgumentTypeHints(CallAst* node, DeclarationPointer
             continue;
         }
         if ( ! matchedNamedParam && kwargsDict ) {
+            DUChainWriteLocker lock;
             kwargsDict->addContentType<Python::UnsureType>(addType);
             parameters.last()->setAbstractType(kwargsDict);
         }
@@ -1168,6 +1154,7 @@ void DeclarationBuilder::assignToSubscript(SubscriptAst* subscript, const Declar
     targetVisitor.visitNode(v);
     auto list = ListType::Ptr::dynamicCast(targetVisitor.lastType());
     if ( list ) {
+        DUChainWriteLocker lock;
         list->addContentType<Python::UnsureType>(element.type);
     }
     auto map = MapType::Ptr::dynamicCast(list);
@@ -1230,7 +1217,7 @@ void DeclarationBuilder::assignToAttribute(AttributeAst* attrib, const Declarati
     if ( ! attributeDeclaration || ! wasEncountered(attributeDeclaration) ) {
         // inject a new attribute into the class type
         DUContext* previousContext = currentContext();
-        bool isAlreadyOpen = contextAlreayOpen(internal);
+        bool isAlreadyOpen = contextAlreadyOpen(internal);
         if ( isAlreadyOpen ) {
             activateAlreadyOpenedContext(internal);
             visitVariableDeclaration<ClassMemberDeclaration>(
@@ -1250,7 +1237,6 @@ void DeclarationBuilder::assignToAttribute(AttributeAst* attrib, const Declarati
                 DUChainWriteLocker lock;
                 previousContext->createUse(dec->ownIndex(), editorFindRange(attrib, attrib));
             }
-            else qCWarning(KDEV_PYTHON_DUCHAIN) << "No declaration created for " << attrib->attribute << "as parent is not a class";
 
             closeInjectedContext();
         }
@@ -1376,19 +1362,27 @@ void DeclarationBuilder::visitAssignment(AssignmentAst* node)
     }
 }
 
+void DeclarationBuilder::visitAnnotationAssignment(AnnotationAssignmentAst* node) {
+    ExpressionVisitor v(currentContext());
+    v.visitNode(node->target);
+    v.visitNode(node->value);
+    auto assignType = v.lastType(); // Never mind aliasing, why annotate that?
+    v.visitNode(node->annotation);
+    assignType = Helper::mergeTypes(assignType, v.lastType());
+    assignToUnknown(node->target, assignType);
+}
+
 void DeclarationBuilder::visitClassDefinition( ClassDefinitionAst* node )
 {
+    visitNodeList(node->decorators);
     const CorrectionHelper::Recursion r(m_correctionHelper->enterClass(node->name->value));
 
     StructureType::Ptr type(new StructureType());
-    
+
     DUChainWriteLocker lock;
     ClassDeclaration* dec = eventuallyReopenDeclaration<ClassDeclaration>(node->name, node->name, NoTypeRequired);
-    lock.unlock();
-    visitDecorators<ClassDeclaration>(node->decorators, dec);
-    lock.lock();
     eventuallyAssignInternalContext();
-    
+
     dec->setKind(KDevelop::Declaration::Type);
     dec->clearBaseClasses();
     dec->setClassType(ClassDeclarationData::Class);
@@ -1453,7 +1447,7 @@ void DeclarationBuilder::visitClassDefinition( ClassDefinitionAst* node )
     dec->setType(type);
 
     openType(type);
-    m_currentClassType = type;
+    m_currentClassTypes.append(type);
 
     // needs to be done here, so the assignment of the internal context happens before visiting the body
     openContextForClassDefinition(node);
@@ -1464,42 +1458,13 @@ void DeclarationBuilder::visitClassDefinition( ClassDefinitionAst* node )
         AstDefaultVisitor::visitNode(_node);
     }
     lock.lock();
-    
+
     closeContext();
+    m_currentClassTypes.removeLast();
     closeType();
     closeDeclaration();
 }
 
-template<typename T> void DeclarationBuilder::visitDecorators(QList< Python::ExpressionAst* > decorators, T* addTo) {
-    foreach ( ExpressionAst* decorator, decorators ) {
-        AstDefaultVisitor::visitNode(decorator);
-        if ( decorator->astType == Ast::CallAstType ) {
-            CallAst* call = static_cast<CallAst*>(decorator);
-            Decorator d;
-            if ( call->function->astType != Ast::NameAstType ) {
-                continue;
-            }
-            d.setName(*static_cast<NameAst*>(call->function)->identifier);
-            foreach ( ExpressionAst* arg, call->arguments ) {
-                if ( arg->astType == Ast::NumberAstType ) {
-                    d.setAdditionalInformation(QString::number(static_cast<NumberAst*>(arg)->value));
-                }
-                else if ( arg->astType == Ast::StringAstType ) {
-                    d.setAdditionalInformation(static_cast<StringAst*>(arg)->value);
-                }
-                break; // we only need the first argument for documentation analysis
-            }
-            addTo->addDecorator(d);
-        }
-        else if ( decorator->astType == Ast::NameAstType ) {
-            NameAst* name = static_cast<NameAst*>(decorator);
-            Decorator d;
-            d.setName(*(name->identifier));
-            addTo->addDecorator(d);
-        }
-    }
-}
-
 void DeclarationBuilder::visitFunctionDefinition( FunctionDefinitionAst* node )
 {
     const CorrectionHelper::Recursion r(m_correctionHelper->enterFunction(node->name->value));
@@ -1514,29 +1479,51 @@ void DeclarationBuilder::visitFunctionDefinition( FunctionDefinitionAst* node )
                                                                                 FunctionDeclarationType);
 
     Q_ASSERT(dec->isFunctionDeclaration());
-    
+
     // check for documentation
     dec->setComment(getDocstring(node->body));
-    
+
     openType(type);
     dec->setInSymbolTable(false);
     dec->setType(type);
-    
+
     lock.unlock();
-    visitDecorators<FunctionDeclaration>(node->decorators, dec);
-    const bool isStatic = Helper::findDecoratorByName<FunctionDeclaration>(dec, "staticmethod");
-    const bool isClassMethod = Helper::findDecoratorByName<FunctionDeclaration>(dec, "classmethod");
-    dec->setStatic(isStatic);
-    dec->setClassMethod(isClassMethod);
+    dec->setStatic(false);
+    dec->setClassMethod(false);
+    dec->setProperty(false);
+    foreach ( auto decorator, node->decorators) {
+        visitNode(decorator);
+        switch (decorator->astType) {
+          case Ast::AttributeAstType: {
+            auto attr = static_cast<AttributeAst*>(decorator)->attribute->value;
+            if ( attr == QStringLiteral("setter") ||
+                 attr == QStringLiteral("getter") ||
+                 attr == QStringLiteral("deleter") )
+                dec->setProperty(true);
+            break;
+          }
+          case Ast::NameAstType: {
+            auto name = static_cast<NameAst*>(decorator)->identifier->value;
+            if ( name == QStringLiteral("staticmethod") )
+                dec->setStatic(true);
+            else if ( name == QStringLiteral("classmethod") )
+                dec->setClassMethod(true);
+            else if ( name == QStringLiteral("property") )
+                dec->setProperty(true);
+            break;
+          }
+          default: {}
+        }
+    }
     visitFunctionArguments(node);
     visitFunctionBody(node);
     lock.lock();
-    
+
     closeDeclaration();
     eventuallyAssignInternalContext();
-    
+
     closeType();
-    
+
     // python methods don't have their parents attributes directly inside them
     if ( eventualParentDeclaration && eventualParentDeclaration->internalContext() && dec->internalContext() ) {
         dec->internalContext()->removeImportedParentContext(eventualParentDeclaration->internalContext());
@@ -1551,19 +1538,19 @@ void DeclarationBuilder::visitFunctionDefinition( FunctionDefinitionAst* node )
             type->setReturnType(currentType<AbstractType>());
         }
         if ( ! type->returnType() ) {
-            type->setReturnType(AbstractType::Ptr(new IntegralType(IntegralType::TypeVoid)));
+            type->setReturnType(AbstractType::Ptr(new NoneType()));
         }
         dec->setType(type);
     }
 
-    if ( ! isStatic ) {
+    if ( ! dec->isStatic() ) {
         DUContext* args = DUChainUtils::getArgumentContext(dec);
         if ( args )  {
             QVector<Declaration*> parameters = args->localDeclarations();
             static IndexedString newMethodName("__new__");
             static IndexedString selfArgumentName("self");
             static IndexedString clsArgumentName("cls");
-            if ( currentContext()->type() == DUContext::Class && ! parameters.isEmpty() && ! isClassMethod ) {
+            if ( currentContext()->type() == DUContext::Class && ! parameters.isEmpty() && ! dec->isClassMethod() ) {
                 QString description;
                 if ( dec->identifier().identifier() == newMethodName
                      && parameters[0]->identifier().identifier() != clsArgumentName )
@@ -1613,7 +1600,6 @@ void DeclarationBuilder::visitFunctionDefinition( FunctionDefinitionAst* node )
         lock.lock();
         if ( v.lastType() && v.isAlias() ) {
             type->setReturnType(Helper::mergeTypes(type->returnType(), v.lastType()));
-            qCDebug(KDEV_PYTHON_DUCHAIN) << "updated function return type to " << type->toString();
             dec->setType(type);
         }
         else if ( ! v.isAlias()) {
@@ -1736,29 +1722,28 @@ void DeclarationBuilder::adjustExpressionsForTypecheck(Python::ExpressionAst* ad
 
 void DeclarationBuilder::visitReturn(ReturnAst* node)
 {
-    // Find the type of the object being "return"ed
-    ExpressionVisitor v(currentContext());
-    v.visitNode(node->value);
-    
-    if ( node->value ) {
-        if ( ! hasCurrentType() ) {
-            DUChainWriteLocker lock;
-            KDevelop::Problem *p = new KDevelop::Problem();
-            p->setFinalLocation(DocumentRange(currentlyParsedDocument(), node->range())); // only mark first line
-            p->setSource(KDevelop::IProblem::SemanticAnalysis);
-            p->setDescription(i18n("Return statement not within function declaration"));
-            ProblemPointer ptr(p);
-            topContext()->addProblem(ptr);
-        }
-        else {
-            TypePtr<FunctionType> t = currentType<FunctionType>();
-            AbstractType::Ptr encountered = v.lastType();
-            DUChainWriteLocker lock;
-            if ( t ) {
-                // Update the containing function's return type
-                t->setReturnType(Helper::mergeTypes(t->returnType(), encountered));
-            }
+    static auto noneType = AbstractType::Ptr(new NoneType());
+
+    if ( auto function = currentType<FunctionType>() ) {
+        // Statements with no explicit value return `None`.
+        auto encountered = noneType;
+        if ( node->value ) {
+            // Find the type of the object being "return"ed
+            ExpressionVisitor v(currentContext());
+            v.visitNode(node->value);
+            encountered = v.lastType();
         }
+        // Update the containing function's return type
+        DUChainWriteLocker lock;
+        function->setReturnType(Helper::mergeTypes(function->returnType(), encountered));
+    } else {
+        DUChainWriteLocker lock;
+        KDevelop::Problem *p = new KDevelop::Problem();
+        p->setFinalLocation(DocumentRange(currentlyParsedDocument(), node->range())); // only mark first line
+        p->setSource(KDevelop::IProblem::SemanticAnalysis);
+        p->setDescription(i18n("Return statement not within function declaration"));
+        ProblemPointer ptr(p);
+        topContext()->addProblem(ptr);
     }
     DeclarationBuilderBase::visitReturn(node);
 }
@@ -1779,7 +1764,6 @@ void DeclarationBuilder::visitArguments( ArgumentsAst* node )
     int parametersCount = node->arguments.length();
     int firstDefaultParameterOffset = parametersCount - defaultParametersCount;
     int currentIndex = 0;
-    qCDebug(KDEV_PYTHON_DUCHAIN) << "arguments:" << node->arguments.size();
     foreach ( ArgAst* arg, node->arguments + node->kwonlyargs ) {
         // Iterate over all the function's arguments, create declarations, and add the arguments
         // to the functions FunctionType.
@@ -1789,17 +1773,16 @@ void DeclarationBuilder::visitArguments( ArgumentsAst* node )
             continue;
         }
 
-        qCDebug(KDEV_PYTHON_DUCHAIN) << "visiting argument:" << arg->argumentName->value;
-
         // Create a variable declaration for the parameter, to be used in the function body.
         Declaration* paramDeclaration = nullptr;
         if ( currentIndex == 1 && workingOnDeclaration->isClassMethod() ) {
             DUChainWriteLocker lock;
             AliasDeclaration* decl = eventuallyReopenDeclaration<AliasDeclaration>(arg->argumentName,
                                                                                    arg, AliasDeclarationType);
-            if ( m_currentClassType ) {
-                qCDebug(KDEV_PYTHON_DUCHAIN) << "setting declaration:" << m_currentClassType->declaration(topContext())->toString();
-                decl->setAliasedDeclaration(m_currentClassType->declaration(currentContext()->topContext()));
+            if ( ! m_currentClassTypes.isEmpty() ) {
+                auto classDecl = m_currentClassTypes.last()->declaration(topContext());
+
+                decl->setAliasedDeclaration(classDecl);
             }
             closeDeclaration();
             paramDeclaration = decl;
@@ -1833,11 +1816,10 @@ void DeclarationBuilder::visitArguments( ArgumentsAst* node )
             workingOnDeclaration->addDefaultParameter(IndexedString("..."));
         }
 
-        qCDebug(KDEV_PYTHON_DUCHAIN) << "is first:" << isFirst << hasCurrentDeclaration() << currentDeclaration();
         if ( isFirst && ! workingOnDeclaration->isStatic() && currentContext() && currentContext()->parentContext() ) {
             DUChainReadLocker lock;
             if ( currentContext()->parentContext()->type() == DUContext::Class ) {
-                argumentType = m_currentClassType.cast<AbstractType>();
+                argumentType = m_currentClassTypes.last().cast<AbstractType>();
                 isFirst = false;
             }
         }
@@ -1845,9 +1827,6 @@ void DeclarationBuilder::visitArguments( ArgumentsAst* node )
         DUChainWriteLocker lock;
         paramDeclaration->setAbstractType(Helper::mergeTypes(paramDeclaration->abstractType(), argumentType));
         type->addArgument(argumentType);
-        if ( argumentType ) {
-            qCDebug(KDEV_PYTHON_DUCHAIN) << "creating argument with type" << argumentType->toString();
-        }
     }
     // Handle *args, **kwargs, and assign them a list / dictionary type.
     if ( node->vararg ) {
