@@ -19,6 +19,7 @@
  */
 
 #include "expressionvisitor.h"
+#include "types/nonetype.h"
 #include "types/indexedcontainer.h"
 #include "declarations/functiondeclaration.h"
 #include "pythonduchainexport.h"
@@ -60,11 +61,10 @@ AbstractType::Ptr ExpressionVisitor::encounterPreprocess(AbstractType::Ptr type)
 ExpressionVisitor::ExpressionVisitor(const DUContext* ctx)
     : DynamicLanguageExpressionVisitor(ctx)
 {
-    ENSURE_CHAIN_NOT_LOCKED
     if ( m_defaultTypes.isEmpty() ) {
         m_defaultTypes.insert(NameConstantAst::True, AbstractType::Ptr(new IntegralType(IntegralType::TypeBoolean)));
         m_defaultTypes.insert(NameConstantAst::False, AbstractType::Ptr(new IntegralType(IntegralType::TypeBoolean)));
-        m_defaultTypes.insert(NameConstantAst::None, AbstractType::Ptr(new IntegralType(IntegralType::TypeVoid)));
+        m_defaultTypes.insert(NameConstantAst::None, AbstractType::Ptr(new NoneType()));
     }
     Q_ASSERT(context());
     Q_ASSERT(context()->topContext());
@@ -76,7 +76,6 @@ ExpressionVisitor::ExpressionVisitor(ExpressionVisitor* parent, const DUContext*
     , m_reportUnknownNames(parent->m_reportUnknownNames)
     , m_scanUntilCursor(parent->m_scanUntilCursor)
 {
-    ENSURE_CHAIN_NOT_LOCKED
     if ( overrideContext ) {
         m_context = overrideContext;
     }
@@ -99,15 +98,21 @@ void ExpressionVisitor::visitAttribute(AttributeAst* node)
     // Like, for B.C where B is an instance of foo, find a property of foo called C.
     DUChainReadLocker lock;
     auto attribute = Helper::accessAttribute(v.lastType(), node->attribute->value, topContext());
-
-    if ( auto resolved = Helper::resolveAliasDeclaration(attribute) ) {
-        encounter(attribute->abstractType(), DeclarationPointer(attribute));
-        setLastIsAlias(dynamic_cast<AliasDeclaration*>(attribute) ||
-                        resolved->isFunctionDeclaration() ||
-                        dynamic_cast<ClassDeclaration*>(resolved));
-    } else {
+    auto resolved = Helper::resolveAliasDeclaration(attribute);
+    if ( ! resolved ) {
         encounterUnknown();
+        return;
     }
+    auto function = dynamic_cast<FunctionDeclaration*>(resolved);
+    if ( function && function->type<FunctionType>() &&
+          function->isProperty() ) {
+        encounter(function->type<FunctionType>()->returnType(), DeclarationPointer(function));
+        return;
+    }
+    encounter(attribute->abstractType(), DeclarationPointer(attribute));
+    setLastIsAlias(function ||
+                    dynamic_cast<AliasDeclaration*>(attribute) ||
+                    dynamic_cast<ClassDeclaration*>(resolved) );
 }
 
 void ExpressionVisitor::visitCall(CallAst* node)
@@ -189,9 +194,8 @@ AbstractType::Ptr ExpressionVisitor::docstringTypeOverride(
         return resultingType;
     };
 
-    QHash< QString, std::function<bool(QStringList, QString)> > knownDecoratorHints;
-    qCDebug(KDEV_PYTHON_DUCHAIN) << "Got function declaration with decorators, checking for list content type...";
-    knownDecoratorHints["getsType"] = [&](QStringList /*arguments*/, QString /*currentHint*/) {
+    QHash< QString, std::function<bool(QStringList, QString)> > knownDocstringHints;
+    knownDocstringHints["getsType"] = [&](QStringList /*arguments*/, QString /*currentHint*/) {
         if ( node->function->astType != Ast::AttributeAstType ) {
             return false;
         }
@@ -199,23 +203,21 @@ AbstractType::Ptr ExpressionVisitor::docstringTypeOverride(
         // when calling foo.bar[3].baz.iteritems(), find the type of "foo.bar[3].baz"
         baseTypeVisitor.visitNode(static_cast<AttributeAst*>(node->function)->value);
         if ( auto t = baseTypeVisitor.lastType().cast<ListType>() ) {
-            qCDebug(KDEV_PYTHON_DUCHAIN) << "Found container, using type";
             docstringType = t->contentType().abstractType();
             return true;
         }
         return false;
     };
 
-    knownDecoratorHints["getsList"] = [&](QStringList /*arguments*/, QString currentHint) {
+    knownDocstringHints["getsList"] = [&](QStringList /*arguments*/, QString currentHint) {
         if ( node->function->astType != Ast::AttributeAstType ) {
             return false;
         }
         ExpressionVisitor baseTypeVisitor(this);
         // when calling foo.bar[3].baz.iteritems(), find the type of "foo.bar[3].baz"
         baseTypeVisitor.visitNode(static_cast<AttributeAst*>(node->function)->value);
-        DUChainWriteLocker lock;
+        DUChainReadLocker lock;
         if ( auto t = baseTypeVisitor.lastType().cast<ListType>() ) {
-            qCDebug(KDEV_PYTHON_DUCHAIN) << "Got container:" << t->toString();
             auto newType = typeObjectForIntegralType<ListType>("list");
             if ( ! newType ) {
                 return false;
@@ -233,42 +235,39 @@ AbstractType::Ptr ExpressionVisitor::docstringTypeOverride(
         }
         return false;
     };
-    knownDecoratorHints["getListOfKeys"] = knownDecoratorHints["getsList"];
+    knownDocstringHints["getListOfKeys"] = knownDocstringHints["getsList"];
 
-    knownDecoratorHints["enumerate"] = [&](QStringList /*arguments*/, QString /*currentHint*/) {
+    knownDocstringHints["enumerate"] = [&](QStringList /*arguments*/, QString /*currentHint*/) {
         if ( node->function->astType != Ast::NameAstType || node->arguments.size() < 1 ) {
             return false;
         }
         ExpressionVisitor enumeratedTypeVisitor(this);
         enumeratedTypeVisitor.visitNode(node->arguments.first());
 
-        DUChainWriteLocker lock;
+        DUChainReadLocker lock;
         auto intType = typeObjectForIntegralType<AbstractType>("int");
         auto enumerated = enumeratedTypeVisitor.lastType();
         docstringType = listOfTuples(intType, Helper::contentOfIterable(enumerated, topContext()));
         return true;
     };
 
-    knownDecoratorHints["getsListOfBoth"] = [&](QStringList /*arguments*/, QString /*currentHint*/) {
-        qCDebug(KDEV_PYTHON_DUCHAIN) << "Got getsListOfBoth decorator, checking container";
+    knownDocstringHints["getsListOfBoth"] = [&](QStringList /*arguments*/, QString /*currentHint*/) {
         if ( node->function->astType != Ast::AttributeAstType ) {
             return false;
         }
         ExpressionVisitor baseTypeVisitor(this);
         // when calling foo.bar[3].baz.iteritems(), find the type of "foo.bar[3].baz"
         baseTypeVisitor.visitNode(static_cast<AttributeAst*>(node->function)->value);
-        DUChainWriteLocker lock;
+        DUChainReadLocker lock;
         if ( auto t = baseTypeVisitor.lastType().cast<MapType>() ) {
-            qCDebug(KDEV_PYTHON_DUCHAIN) << "Got container:" << t->toString();
             docstringType = listOfTuples(t->keyType().abstractType(), t->contentType().abstractType());
             return true;
         }
         return false;
     };
 
-    knownDecoratorHints["returnContentEqualsContentOf"] = [&](QStringList arguments, QString /*currentHint*/) {
-        int argNum = ! arguments.isEmpty() ? arguments.at(0).toInt() : 0;
-        qCDebug(KDEV_PYTHON_DUCHAIN) << "Found argument dependent decorator, checking argument type" << argNum;
+    knownDocstringHints["returnContentEqualsContentOf"] = [&](QStringList arguments, QString /*currentHint*/) {
+        const int argNum = ! arguments.isEmpty() ? (int) arguments.at(0).toUInt() : 0;
         if ( argNum >= node->arguments.length() ) {
             return false;
         }
@@ -296,19 +295,17 @@ AbstractType::Ptr ExpressionVisitor::docstringTypeOverride(
         return false;
     };
 
-    foreach ( const QString& currentHint, knownDecoratorHints.keys() ) {
+    foreach ( const QString& currentHint, knownDocstringHints.keys() ) {
         QStringList arguments;
         if ( ! Helper::docstringContainsHint(docstring, currentHint, &arguments) ) {
             continue;
         }
         // If the hint word appears in the docstring, run the evaluation function.
-        if ( knownDecoratorHints[currentHint](arguments, currentHint) ) {
+        if ( knownDocstringHints[currentHint](arguments, currentHint) ) {
             // We indeed found something, so we're done.
-            return docstringType;
+            break;
         }
     }
-
-    // if none of the above decorator-finding methods worked, just use the ordinary return type.
     return docstringType;
 }
 
@@ -346,10 +343,10 @@ void ExpressionVisitor::visitSubscript(SubscriptAst* node)
             }
             if ( number ) {
                 int sliceIndex = number->value * ( invert ? -1 : 1 );
-                if ( sliceIndex < 0 && sliceIndex + indexed->typesCount() > 0 ) {
+                if ( sliceIndex < 0 && sliceIndex + indexed->typesCount() >= 0 ) {
                     sliceIndex += indexed->typesCount();
                 }
-                if ( sliceIndex < indexed->typesCount() && sliceIndex >= 0 ) {
+                if ( sliceIndex >= 0 && sliceIndex < indexed->typesCount() ) {
                     result = Helper::mergeTypes(result, indexed->typeAt(sliceIndex).abstractType());
                     continue;
                 }
